//
// shatbox (C) 2002 Matthew Conte (matt@conte.com)
//
//
// nes.h
//
// main NES emulation
//
// $Id: $
//

#ifndef _NES_H_
#define _NES_H_

#include "typedefs.h"
#include "machine.h"


class System;
class eventHandler;
class mmc;
class cart;
class cpu;
class apu;
class ppu;
class input;
class NES : public machine
{
public:
   NES(const char *pFilename);
   virtual ~NES();

   virtual void run();
   virtual void reset(bool hardReset = false);
   virtual void setThrottle(bool throttle);
   virtual void triggerInput(int device, int data, bool state);

   // TODO: temp hack
   void  setColor(int index, int r, int g, int b, int value = -1);

   virtual void saveState();
   virtual void loadState();

   cpu   *getCPU() const;
   apu   *getAPU() const;
   ppu   *getPPU() const;

private:
   void  frame();

   cpu   *mpCPU;
   apu   *mpAPU;
   ppu   *mpPPU;
   mmc   *mpMMC;
   
   cart  *mpCart;

   input *mpInput;

   eventHandler *mpEventHandler;

   System *mpSystem;

   const char *mpROMFilename;

   float mFramePeriod;
};

inline cpu *NES::getCPU() const
{
   return mpCPU;
}

inline apu *NES::getAPU() const
{
   return mpAPU;
}

inline ppu *NES::getPPU() const
{
   return mpPPU;
}

#endif // !_NES_H_
