/*
// shatbox (C) 2002 Matthew Conte (matt@conte.com)
**
**
** nes6502.h
**
** NES custom 6502 CPU
**
** $Id: $
*/

#ifndef _NES6502_H_
#define _NES6502_H_

#include "typedefs.h"

#define  NES6502_ADDRSPACE 0x10000  /* 64kB */
#define  NES6502_NUMBANKS  16
#define  NES6502_BANKSIZE  (NES6502_ADDRSPACE / NES6502_NUMBANKS)

/* defining this makes the cpu emulator behave like a real 6502,
** rather than like the custom 6502 in the NES (N2A03), which has:
** - decimal mode disabled in ADC / SBC
** - 2kB of RAM is mirrored from $0000 - $1FFF
** - $8000-$FFFF is always banked memory for reads
*/
/*#define  NES6502_STANDARD*/


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef uint8 (*nes6502_read)(void *userdata, uint32 offset);
typedef void  (*nes6502_write)(void *userdata, uint32 offset, uint8 value);

typedef struct nes6502_memread_s
{
   uint32         min_range, max_range;
   nes6502_read   read_func;
   void           *userdata;
} nes6502_memread;

typedef struct nes6502_memwrite_s
{
   uint32         min_range, max_range;
   nes6502_write  write_func;
   void           *userdata;
} nes6502_memwrite;

typedef struct nes6502_context_s
{
   /* memory page pointers */
   uint8    *mem_page[NES6502_NUMBANKS];

   /* user callback, for when cpu hits a JAM instruction */
   void     (*jam_callback)(struct nes6502_context_s *context);

   /* memory read/write handlers */
   nes6502_memread   *read_handler;
   nes6502_memwrite  *write_handler;

   /* private data */
   int32    total_cycles;
   int32    burn_cycles;
   int32    remaining_cycles;

   uint32   pc_reg;
   uint8    a_reg, p_reg;
   uint8    x_reg, y_reg;
   uint8    s_reg;

   bool     jammed;
   bool     int_pending;
} nes6502_context;

/* Functions which govern the 6502's execution */
extern void    nes6502_reset(void);
extern int     nes6502_execute(int total_cycles);
extern void    nes6502_nmi(void);
extern void    nes6502_irq(void);
extern void    nes6502_clearirq(void);
extern uint8   nes6502_getbyte(uint32 address);
extern uint32  nes6502_getcycles(bool reset_flag);
extern void    nes6502_burn(int cycles);
extern void    nes6502_release(void);

/* Context get/set */
extern void    nes6502_setcontext(nes6502_context *cpu);
extern void    nes6502_getcontext(nes6502_context *cpu);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _NES6502_H_ */
